/**
 * Kode-fil for stoppested-klassen
 *
 * @file stoppested.cpp
 * @author Sondre Sand & Frederik Simonsen
 */
#include <iomanip>
#include <iostream>
#include <string>
#include <fstream>
#include <vector>
#include <algorithm> // find, distance
#include "stoppested.h"
#include "stoppesteder.h"
using namespace std;

extern Stoppesteder gStoppestederBase; // Henter inn globalt Stopperstederobjekt

/**
 * Run-time constructor, ifm. menyvalg S N.
 * Kopierer medsendt unikt navn til å bli navn
 * på stoppestedet
 *
 * @param nvn - Stoppestedets unike navn som er duplikatsjekket
 */
Stoppested::Stoppested(const string nvn) {
    navn = nvn;
}

/**
 * Constructor fra fil
 * Setter et stoppesteds navn, leser antall naboer
 * og legger til eventuelle naboer og deres tid inn i datastrukturen
 * 
 * @param nvn Stoppestedets navn (første post)
 * @param inn Filen det leses fra
 */
Stoppested::Stoppested(const std::string nvn, ifstream & inn) {
    int antNaboer,
        naboNr,
        tidTilN;

    navn = nvn; // Setter stoppestedets navn
    inn >> antNaboer; inn.ignore(); // Leser et stoppesteds antall naboer
    for (int i = 0; i < antNaboer; i++) { // For hver nabo
        inn >> naboNr >> tidTilN; // Leser deres unike nummer og tid til
        nabostopper.push_back(naboNr); // Nr legges bakerst i vektor
        tidtilnabo.push_back(tidTilN); // Tid legges bakerst i vektor
    }

    if (antNaboer != 0) // Må ha denne, ellers kutter første tegnet i navnet
        inn.ignore(); // Forkaster enter hvis det finnes naboer
}

/**
 * Sjekker om et Stoppested har en gitt nabo
 * 
 * @param nr Evt. nabos unike nummer
 * @return true Om stoppestedet har denne som nabo
 * @return false Om stoppestedet ikke har denne som nabo
 */
bool Stoppested::finnesNabo(const int nr) {
    // Prøver å finne evt. nabo:
    auto it = find(nabostopper.begin(), nabostopper.end(), nr);
    if (it != nabostopper.end()) { // Hvis nabostoppen finnes i vector:
        return true; // Returnerer at nabo er funnet
    } // Hvis ingen funn:
    return false; // Returnerer at nabo ikke er funnet
}

/**
 * Henter en nabos faktiske indeks i egen vector
 * 
 * @param nr Naboens unike nummer som skal finnes
 * @return int Avstand mellom start på vector og faktisk funn av nabo
 *             som = deres indeks i vectoren.
 *         -1  Skal ikke skje, men returner -1 hvis ikke funn
 */
int Stoppested::hentNaboIndeks(const int nr) {
    // Går gjennom hele vectoren, forsøker å finne et stoppesteds nummer
    auto it = find(nabostopper.begin(), nabostopper.end(), nr);
    if (it != nabostopper.end()) { // Funn, returnerer indeks til nummeret:
        return distance(nabostopper.begin(), it);
    }
    return -1; // Skal ikke skje
}

/**
 * Henter tid mellom eget stoppested og en nabo
 * 
 * @param nr - Naboens indeks i egen datastruktur
 * @return Faktiske tid mellom eget stoppested og nabo
 */
int Stoppested::hentNaboTid(const int nr) {
    return tidtilnabo[nr];
}

/**
 * Returnerer et Stoppesteds unike navn
 *
 * @return Stoppestedets navn
 */
string Stoppested::hentStoppestedNavn() {
    return navn;
}

/**
 * Setter et nabostoppestedets unike nr inn i egen vector
 * 
 * @param nr Naboens indeks
 */
void Stoppested::settNaboIndeks(const int nr) {
    nabostopper.push_back(nr+1); // +1 her for faktiske stoppestedNr
}

/**
 * Setter tid mellom et stoppested og en nabo i egen vector 
 * 
 * @param nr Tid mellom et stoppested og en nabo
 */
void Stoppested::settNaboTid(const int nr) {
    tidtilnabo.push_back(nr); // Legges bakerst i egen vector
}

/**
 * Skriver ut Stoppestedets navn på skjerm ifm. S A
 * Formaterer utskrift til å skrive navn på 20 "plasser"
 */
void Stoppested::skrivData() {
    cout << setw(20) << navn;
}

/**
 * Skriver ut et stoppesteds naboer og tid til nabostoppestedene
 * 
 * @see Stoppesteder::hentNavnVhaIndeks(...)
 */
void Stoppested::skrivNaboerOgTid() {
    if (nabostopper.size() > 0) { // Hvis stoppestedet har naboer:
        for (int i = 0; i < nabostopper.size(); i++) { // For hver nabo:
            cout << setw(25) // Skriver ut naboens navn
                 << gStoppestederBase.hentNavnVhaIndeks(nabostopper[i])
                 << "  (" << setw(3) << tidtilnabo[i] 
                 << " min mellom)\n"; // Skriver ut tid mellom nabo og stopp
        }
    } else // Hvis stoppestedet ikke har naboer:
        cout << "Dette stoppestedet har ingen nabostopp.\n\n";

}

/**
 * Skriver et stoppesteds datastruktur til fil
 * 
 * @param ut Filen det skrives til
 */
void Stoppested::skrivTilFil(ofstream & ut) {
    ut << navn << '\n'; // Skriver ut navn og antall naboer:
    ut << nabostopper.size() << '\n';
    for (int i = 0; i < nabostopper.size(); i++) { // For hver nabo:
        ut << nabostopper[i] << ' '; // Skriver deres unike nr
        ut << tidtilnabo[i] << '\n'; // og tid mellom
    }
}